"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSerializer = exports.asString = void 0;
var generate = function (schema, fn) {
    if (fn === void 0) { fn = []; }
    switch (schema.type) {
        case 'object': {
            var code = '(function(v){return "';
            var keys = Object.keys(schema.properties);
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                code += "" + (i > 0 ? '+",' : '{') + JSON.stringify(JSON.stringify(key)).slice(1, -1) + ":\"+" + generate(schema.properties[key], fn) + "(v[" + JSON.stringify(key) + "])";
            }
            code += '+"}"})';
            var f = 'f' + fn.length;
            fn.push(code);
            return f;
        }
        case 'array': {
            var type = generate(schema.items, fn);
            return '(function(v){' +
                'var i,j="[",l=v.length;' +
                'for(i=0;i<l;i++){' +
                'if(i>0)j+=",";' +
                'j+=' + type + "(v[i])" +
                '}' +
                'return j+"]"})';
        }
        case 'json': return 'f0';
        case 'string_escaped': return 'f1';
        case 'string': return 'f2';
        default: return '';
    }
};
function asStringSmall(str) {
    var result = '';
    var last = 0;
    var found = false;
    var surrogateFound = false;
    var l = str.length;
    var point = 255;
    for (var i = 0; i < l && point >= 32; i++) {
        point = str.charCodeAt(i);
        if (point >= 0xD800 && point <= 0xDFFF) {
            surrogateFound = true;
        }
        if (point === 34 || point === 92) {
            result += str.slice(last, i) + '\\';
            last = i;
            found = true;
        }
    }
    if (!found) {
        result = str;
    }
    else {
        result += str.slice(last);
    }
    return ((point < 32) || (surrogateFound === true)) ? JSON.stringify(str) : '"' + result + '"';
}
function asString(str) {
    if (str.length < 42) {
        return asStringSmall(str);
    }
    else {
        return JSON.stringify(str);
    }
}
exports.asString = asString;
exports.createSerializer = function (schema) {
    var fn = [
        'JSON.stringify',
        'function(s){return \'"\'+s+\'"\'}',
        asString.toString(),
    ];
    var code = generate(schema, fn);
    var header = '';
    for (var i = 0; i < fn.length; i++)
        header += "var f" + i + "=" + fn[i] + ";\n";
    code = header + 'return function(val){return ""+(' + code + '(val))}';
    code = '(function(){' + code + '})()';
    return eval(code);
};
